@@ -456,7 +456,7 @@ static MSFilterMethod player_methods[]={
 	{	0,			NULL		}
 };
 
-#ifdef WIN32
+#ifdef _WIN32
 
 MSFilterDesc ms_file_player_desc={
 	MS_FILE_PLAYER_ID,
