@@ -36,12 +36,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #if !defined(_WIN32_WCE)
 #include <sys/types.h>
 #endif
-#ifndef WIN32
+#ifndef _WIN32
 #include <dirent.h>
 #else
 #ifndef PACKAGE_PLUGINS_DIR
-#if defined(WIN32) || defined(_WIN32_WCE)
-#if WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
+#if defined(_WIN32) || defined(_WIN32_WCE)
+#ifdef MS2_WINDOWS_DESKTOP
 #define PACKAGE_PLUGINS_DIR "lib\\mediastreamer\\plugins\\"
 #else
 #define PACKAGE_PLUGINS_DIR "."
@@ -108,6 +108,25 @@ void ms_factory_set_cpu_count(MSFactory *obj, unsigned int c) {
 	obj->cpu_count = c;
 }
 
+void ms_factory_add_platform_tag(MSFactory *obj, const char *tag) {
+	if ((tag == NULL) || (tag[0] == '\0')) return;
+	if (ms_list_find_custom(obj->platform_tags, (MSCompareFunc)strcasecmp, tag) == NULL) {
+		obj->platform_tags = ms_list_append(obj->platform_tags, ms_strdup(tag));
+	}
+}
+
+MSList * ms_factory_get_platform_tags(MSFactory *obj) {
+	return obj->platform_tags;
+}
+
+char * ms_factory_get_platform_tags_as_string(MSFactory *obj) {
+	return ms_tags_list_as_string(obj->platform_tags);
+}
+
+struct _MSVideoPresetsManager * ms_factory_get_video_presets_manager(MSFactory *factory) {
+	return factory->video_presets_manager;
+}
+
 static int compare_stats_with_name(const MSFilterStats *stat, const char *name){
 	return strcmp(stat->name,name);
 }
@@ -134,15 +153,18 @@ MSFactory *ms_factory_get_fallback(void){
 void ms_factory_init(MSFactory *obj){
 	int i;
 	long num_cpu=1;
-	char *debug_log_enabled;
-#ifdef WIN32
+	char *debug_log_enabled = NULL;
+	char *tags;
+#ifdef _WIN32
 	SYSTEM_INFO sysinfo;
 #endif
 
 #if defined(ENABLE_NLS)
 	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
 #endif
+#ifndef MS2_WINDOWS_UNIVERSAL
 	debug_log_enabled=getenv("MEDIASTREAMER_DEBUG");
+#endif
 	if (debug_log_enabled!=NULL && (strcmp("1",debug_log_enabled)==0) ){
 		ortp_set_log_level_mask(ORTP_MESSAGE|ORTP_WARNING|ORTP_ERROR|ORTP_FATAL);
 	}
@@ -153,7 +175,7 @@ void ms_factory_init(MSFactory *obj){
 		ms_factory_register_filter(obj,ms_base_filter_descs[i]);
 	}
 
-#ifdef WIN32 /*fixme to be tested*/
+#ifdef _WIN32 /*fixme to be tested*/
 	GetNativeSystemInfo( &sysinfo );
 
 	num_cpu = sysinfo.dwNumberOfProcessors;
@@ -166,7 +188,43 @@ void ms_factory_init(MSFactory *obj){
 #endif
 	ms_factory_set_cpu_count(obj,num_cpu);
 	ms_factory_set_mtu(obj,MS_MTU_DEFAULT);
-	ms_message("ms_factory_init() done");
+#ifdef _WIN32
+	ms_factory_add_platform_tag(obj, "win32");
+#ifdef MS2_WINDOWS_PHONE
+	ms_factory_add_platform_tag(obj, "windowsphone");
+#endif
+#ifdef MS2_WINDOWS_UNIVERSAL
+	ms_factory_add_platform_tag(obj, "windowsuniversal");
+#endif
+#endif
+#ifdef __APPLE__
+	ms_factory_add_platform_tag(obj, "apple");
+#endif
+#ifdef __linux
+	ms_factory_add_platform_tag(obj, "linux");
+#endif
+#ifdef __QNX__
+	ms_factory_add_platform_tag(obj, "qnx");
+#endif
+#ifdef ANDROID
+	ms_factory_add_platform_tag(obj, "android");
+#endif
+#ifdef TARGET_OS_IPHONE
+	ms_factory_add_platform_tag(obj, "ios");
+#endif
+#if defined(__arm__) || defined(_M_ARM)
+	ms_factory_add_platform_tag(obj, "arm");
+#else
+	ms_factory_add_platform_tag(obj, "x86");
+#endif
+#if defined(ANDROID) || (TARGET_OS_IPHONE == 1) || defined(__arm__) || defined(_M_ARM)
+	ms_factory_add_platform_tag(obj, "embedded");
+#else
+	ms_factory_add_platform_tag(obj, "desktop");
+#endif
+	tags = ms_factory_get_platform_tags_as_string(obj);
+	ms_message("ms_factory_init() done: platform_tags=%s", tags);
+	ms_free(tags);
 }
 
 MSFactory *ms_factory_create_fallback(void){
@@ -193,6 +251,8 @@ void ms_factory_destroy(MSFactory *factory){
 	factory->desc_list=ms_list_free(factory->desc_list);
 	ms_list_for_each(factory->stats_list,ms_free);
 	factory->stats_list=ms_list_free(factory->stats_list);
+	ms_list_for_each(factory->platform_tags, ms_free);
+	factory->platform_tags = ms_list_free(factory->platform_tags);
 	if (factory->plugins_dir) ms_free(factory->plugins_dir);
 	ms_free(factory);
 	if (factory==fallback_factory) fallback_factory=NULL;
@@ -204,6 +264,8 @@ void ms_factory_register_filter(MSFactory* factory, MSFilterDesc* desc ) {
 	if (desc->id==MS_FILTER_NOT_SET_ID){
 		ms_fatal("MSFilterId for %s not set !",desc->name);
 	}
+	desc->flags|=MS_FILTER_IS_ENABLED; /*by default a registered filter is enabled*/
+
 	/*lastly registered encoder/decoders may replace older ones*/
 	factory->desc_list=ms_list_prepend(factory->desc_list,desc);
 }
@@ -270,8 +332,9 @@ MSFilterDesc * ms_factory_get_encoder(MSFactory* factory, const char *mime){
 	MSList *elem;
 	for (elem=factory->desc_list;elem!=NULL;elem=ms_list_next(elem)){
 		MSFilterDesc *desc=(MSFilterDesc*)elem->data;
-		if ((desc->category==MS_FILTER_ENCODER || desc->category==MS_FILTER_ENCODING_CAPTURER) &&
-			strcasecmp(desc->enc_fmt,mime)==0){
+		if ((desc->flags & MS_FILTER_IS_ENABLED)
+			&& (desc->category==MS_FILTER_ENCODER || desc->category==MS_FILTER_ENCODING_CAPTURER)
+			&& strcasecmp(desc->enc_fmt,mime)==0){
 			return desc;
 		}
 	}
@@ -282,8 +345,9 @@ MSFilterDesc * ms_factory_get_decoder(MSFactory* factory, const char *mime){
 	MSList *elem;
 	for (elem=factory->desc_list;elem!=NULL;elem=ms_list_next(elem)){
 		MSFilterDesc *desc=(MSFilterDesc*)elem->data;
-		if ((desc->category==MS_FILTER_DECODER || desc->category==MS_FILTER_DECODER_RENDERER) &&
-			strcasecmp(desc->enc_fmt,mime)==0){
+		if ((desc->flags & MS_FILTER_IS_ENABLED)
+			&& (desc->category==MS_FILTER_DECODER || desc->category==MS_FILTER_DECODER_RENDERER)
+			&& strcasecmp(desc->enc_fmt,mime)==0){
 			return desc;
 		}
 	}
@@ -331,7 +395,7 @@ MSFilter *ms_factory_create_filter(MSFactory* factory, MSFilterId id){
 	return NULL;
 }
 
-MSFilterDesc *ms_factory_lookup_filter_by_name(MSFactory* factory, const char *filter_name){
+MSFilterDesc *ms_factory_lookup_filter_by_name(const MSFactory* factory, const char *filter_name){
 	MSList *elem;
 	for (elem=factory->desc_list;elem!=NULL;elem=ms_list_next(elem)){
 		MSFilterDesc *desc=(MSFilterDesc*)elem->data;
@@ -426,7 +490,7 @@ typedef void (*init_func_t)(MSFactory *);
 
 int ms_factory_load_plugins(MSFactory *factory, const char *dir){
 	int num=0;
-#if defined(WIN32) && !defined(_WIN32_WCE)
+#if defined(_WIN32) && !defined(_WIN32_WCE)
 	WIN32_FIND_DATA FileData;
 	HANDLE hSearch;
 	char szDirPath[1024];
@@ -435,13 +499,17 @@ int ms_factory_load_plugins(MSFactory *factory, const char *dir){
 #endif
 	char szPluginFile[1024];
 	BOOL fFinished = FALSE;
-	const char *tmp=getenv("DEBUG");
-	BOOL debug=(tmp!=NULL && atoi(tmp)==1);
+	const char *tmp = NULL;
+	BOOL debug = FALSE;
+#ifndef MS2_WINDOWS_UNIVERSAL
+	tmp = getenv("DEBUG");
+#endif
+	debug = (tmp != NULL && atoi(tmp) == 1);
 
 	snprintf(szDirPath, sizeof(szDirPath), "%s", dir);
 
 	// Start searching for .dll files in the current directory.
-#if WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
+#ifdef MS2_WINDOWS_DESKTOP
 	snprintf(szDirPath, sizeof(szDirPath), "%s\\*.dll", dir);
 #else
 	snprintf(szDirPath, sizeof(szDirPath), "%s\\libms*.dll", dir);
@@ -462,7 +530,7 @@ int ms_factory_load_plugins(MSFactory *factory, const char *dir){
 	while (!fFinished)
 	{
 		/* load library */
-#if WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
+#ifdef MS2_WINDOWS_DESKTOP
 		UINT em=0;
 #endif
 		HINSTANCE os_handle;
@@ -475,7 +543,7 @@ int ms_factory_load_plugins(MSFactory *factory, const char *dir){
 #else
 		snprintf(szPluginFile, sizeof(szPluginFile), "%s\\%s", szDirPath, FileData.cFileName);
 #endif
-#if WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
+#ifdef MS2_WINDOWS_DESKTOP
 		if (!debug) em = SetErrorMode (SEM_FAILCRITICALERRORS);
 
 #ifdef UNICODE
@@ -614,11 +682,11 @@ int ms_factory_load_plugins(MSFactory *factory, const char *dir){
 }
 
 void ms_factory_uninit_plugins(MSFactory *factory){
-#if defined(WIN32)
+#if defined(_WIN32)
 	MSList *elem;
 #endif
 
-#if defined(WIN32)
+#if defined(_WIN32)
 	for(elem=factory->ms_plugins_loaded_list;elem!=NULL;elem=elem->next)
 	{
 		HINSTANCE handle=(HINSTANCE )elem->data;
@@ -652,13 +720,17 @@ void ms_factory_set_plugins_dir(MSFactory *obj, const char *path) {
 		obj->plugins_dir = ms_strdup(path);
 }
 
-struct _MSEventQueue *ms_factory_get_event_queue(MSFactory *obj){
+struct _MSEventQueue *ms_factory_create_event_queue(MSFactory *obj) {
 	if (obj->evq==NULL){
 		obj->evq=ms_event_queue_new();
 	}
 	return obj->evq;
 }
 
+struct _MSEventQueue *ms_factory_get_event_queue(MSFactory *obj){
+	return obj->evq;
+}
+
 /*this function is for compatibility, when event queues were created by the application*/
 void ms_factory_set_event_queue(MSFactory *obj, MSEventQueue *evq){
 	obj->evq=evq;
@@ -745,3 +817,41 @@ const MSFmtDescriptor * ms_factory_get_video_format(MSFactory *obj, const char *
 	return ms_factory_get_format(obj,&tmp);
 }
 
+int ms_factory_enable_filter_from_name(MSFactory *factory, const char *name, bool_t enable) {
+	MSFilterDesc *desc=ms_factory_lookup_filter_by_name(factory,name);
+	if (!desc) {
+		ms_error("Cannot enable/disable unknown filter [%s] on factory [%p]",name,factory);
+		return -1;
+	}
+	if (enable) desc->flags |= MS_FILTER_IS_ENABLED;
+	else desc->flags &= ~MS_FILTER_IS_ENABLED;
+	ms_message("Filter [%s]  %s on factory [%p]",name,(enable ? "enabled" : "disabled"),factory);
+	return 0;
+}
+
+bool_t ms_factory_filter_from_name_enabled(const MSFactory *factory, const char *name) {
+	MSFilterDesc *desc=ms_factory_lookup_filter_by_name(factory,name);
+	if (!desc) {
+		ms_error("Cannot get enable/disable state for unknown filter [%s] on factory [%p]",name,factory);
+		return FALSE;
+	}
+	return !!(desc->flags & MS_FILTER_IS_ENABLED);
+}
+
+#ifdef ANDROID
+#include "sys/system_properties.h"
+#include <jni.h>
+
+JNIEXPORT jint JNICALL Java_org_linphone_mediastream_MediastreamerAndroidContext_enableFilterFromNameImpl(JNIEnv* env,  jobject obj, jstring jname, jboolean enable) {
+	const char *mime = jname ? (*env)->GetStringUTFChars(env, jname, NULL) : NULL;
+	int result = ms_factory_enable_filter_from_name(ms_factory_get_fallback(),mime,enable);
+	(*env)->ReleaseStringUTFChars(env, jname, mime);
+	return result;
+}
+JNIEXPORT jboolean JNICALL Java_org_linphone_mediastream_MediastreamerAndroidContext_filterFromNameEnabledImpl(JNIEnv* env, jobject obj, jstring jname) {
+	const char *mime = jname ? (*env)->GetStringUTFChars(env, jname, NULL) : NULL;
+	jboolean result = ms_factory_filter_from_name_enabled(ms_factory_get_fallback(),mime);
+	(*env)->ReleaseStringUTFChars(env, jname, mime);
+	return result;
+}
+#endif
