@@ -44,7 +44,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include <sys/stat.h>
 
-#ifdef WIN32
+#ifdef _WIN32
 #include <fcntl.h>
 #include <sys/types.h>
 #include <io.h>
@@ -84,9 +84,9 @@ static mblk_t *jpeg2yuv(uint8_t *jpgbuf, int bufsize, MSVideoSize *reqsize){
 	}
 	ret=ms_yuv_buf_alloc(&dest, reqsize->width,reqsize->height);
 	/* not using SWS_FAST_BILINEAR because it doesn't play well with
-	 * av_context.pix_fmt set to PIX_FMT_YUVJ420P by jpeg decoder */
+	 * av_context.pix_fmt set to AV_PIX_FMT_YUVJ420P by jpeg decoder */
 	sws_ctx=sws_getContext(av_context.width,av_context.height,av_context.pix_fmt,
-		reqsize->width,reqsize->height,PIX_FMT_YUV420P,SWS_BILINEAR,
+		reqsize->width,reqsize->height,AV_PIX_FMT_YUV420P,SWS_BILINEAR,
                 NULL, NULL, NULL);
 	if (sws_ctx==NULL) {
 		ms_error("jpeg2yuv: ms_sws_getContext() failed.");
@@ -169,8 +169,8 @@ static mblk_t *jpeg2yuv(uint8_t *jpgbuf, int bufsize, MSVideoSize *reqsize){
 
 
 
-mblk_t *ms_load_jpeg_as_yuv(const char *jpgpath, MSVideoSize *reqsize){
-#if defined(WIN32)
+static mblk_t *_ms_load_jpeg_as_yuv(const char *jpgpath, MSVideoSize *reqsize){
+#if defined(_WIN32)
 	mblk_t *m=NULL;
 	DWORD st_sizel;
 	DWORD st_sizeh;
@@ -183,7 +183,7 @@ mblk_t *ms_load_jpeg_as_yuv(const char *jpgpath, MSVideoSize *reqsize){
 #else
 	const char *wUnicode = jpgpath;
 #endif
-#if !WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
+#ifndef MS2_WINDOWS_DESKTOP
 	fd = CreateFile2(wUnicode, GENERIC_READ, FILE_SHARE_READ, OPEN_EXISTING, NULL);
 #else
 	fd = CreateFile(wUnicode, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
@@ -194,7 +194,7 @@ mblk_t *ms_load_jpeg_as_yuv(const char *jpgpath, MSVideoSize *reqsize){
 	}
 	st_sizel=0;
 	st_sizeh=0;
-#if !WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
+#ifndef MS2_WINDOWS_DESKTOP
 	{
 		WIN32_FILE_ATTRIBUTE_DATA attr_data;
 		GetFileAttributesEx(wUnicode, GetFileExInfoStandard, &attr_data);
@@ -276,6 +276,51 @@ mblk_t *ms_load_jpeg_as_yuv(const char *jpgpath, MSVideoSize *reqsize){
 #endif
 }
 
+#ifdef MS2_WINDOWS_UNIVERSAL
+typedef mblk_t * (*jpeg2yuv_routine_t)(const char *, MSVideoSize *);
+
+mblk_t * _ms_winrt_load_jpeg_as_yuv(const char *jpgpath, MSVideoSize *reqsize) {
+	mblk_t *result = NULL;
+	HMODULE h = LoadPackagedLibrary(L"mswinrtjpeg2yuv.dll", 0);
+	if (h == NULL) {
+		ms_error("Cannot load mswinrtjpeg2yuv.dll to convert jpeg file to YUV");
+	}
+	else {
+		jpeg2yuv_routine_t routine = (jpeg2yuv_routine_t)GetProcAddress(h, "winrtjpeg2yuv");
+		if (routine == NULL) {
+			ms_error("Cannot find winrtjpeg2yuv routine in mswinrtjpeg2yuv.dll");
+		}
+		else {
+			result = routine(jpgpath, reqsize);
+		}
+	}
+	return result;
+}
+#endif
+
+static mblk_t * generate_black_yuv_frame(MSVideoSize *reqsize) {
+	MSPicture dest;
+	mblk_t *m = ms_yuv_buf_alloc(&dest, reqsize->width, reqsize->height);
+	int ysize = dest.w * dest.h;
+	int usize = ysize / 4;
+	memset(dest.planes[0], 16, ysize);
+	memset(dest.planes[1], 128, usize);
+	memset(dest.planes[2], 128, usize);
+	return m;
+}
+
+mblk_t *ms_load_jpeg_as_yuv(const char *jpgpath, MSVideoSize *reqsize) {
+	mblk_t *m = NULL;
+	if (jpgpath != NULL) {
+#ifdef MS2_WINDOWS_UNIVERSAL
+		m = _ms_winrt_load_jpeg_as_yuv(jpgpath, reqsize);
+#else
+		m = _ms_load_jpeg_as_yuv(jpgpath, reqsize);
+#endif
+	}
+	if (m == NULL) m = generate_black_yuv_frame(reqsize);
+	return m;
+}
 
 
 #ifndef PACKAGE_DATA_DIR
@@ -337,8 +382,8 @@ void static_image_uninit(MSFilter *f){
 
 void static_image_preprocess(MSFilter *f){
 	SIData *d=(SIData*)f->data;
-	if (d->pic==NULL && d->nowebcamimage){
-		d->pic=ms_load_jpeg_as_yuv(d->nowebcamimage,&d->vsize);
+	if (d->pic==NULL) {
+		d->pic = ms_load_jpeg_as_yuv(d->nowebcamimage, &d->vsize);
 	}
 }
 
