@@ -36,6 +36,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/msitc.h"
 #include "mediastreamer2/msvaddtx.h"
 #include "mediastreamer2/msgenericplc.h"
+#include "mediastreamer2/mseventqueue.h"
 #include "private.h"
 
 #ifdef ANDROID
@@ -44,14 +45,15 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #include <sys/types.h>
 
-#ifndef WIN32
+#ifndef _WIN32
 	#include <sys/socket.h>
 	#include <netdb.h>
 #endif
 
 static void configure_av_recorder(AudioStream *stream);
 static void configure_decoder(AudioStream *stream, PayloadType *pt, int sample_rate, int nchannels);
-static void audio_stream_configure_resampler(MSFilter *resampler,MSFilter *from, MSFilter *to, PayloadType *pt);
+static void audio_stream_configure_resampler(AudioStream *st, MSFilter *resampler,MSFilter *from, MSFilter *to);
+static void audio_stream_set_rtp_output_gain_db(AudioStream *stream, float gain_db);
 
 static void audio_stream_free(AudioStream *stream) {
 	media_stream_free(&stream->ms);
@@ -63,6 +65,8 @@ static void audio_stream_free(AudioStream *stream) {
 	if (stream->volrecv!=NULL) ms_filter_destroy(stream->volrecv);
 	if (stream->volsend!=NULL) ms_filter_destroy(stream->volsend);
 	if (stream->equalizer!=NULL) ms_filter_destroy(stream->equalizer);
+	if (stream->read_decoder != NULL) ms_filter_destroy(stream->read_decoder);
+	if (stream->write_encoder != NULL) ms_filter_destroy(stream->write_encoder);
 	if (stream->read_resampler!=NULL) ms_filter_destroy(stream->read_resampler);
 	if (stream->write_resampler!=NULL) ms_filter_destroy(stream->write_resampler);
 	if (stream->dtmfgen_rtp!=NULL) ms_filter_destroy(stream->dtmfgen_rtp);
@@ -80,13 +84,14 @@ static void audio_stream_free(AudioStream *stream) {
 	if (stream->vaddtx) ms_filter_destroy(stream->vaddtx);
 	if (stream->outbound_mixer) ms_filter_destroy(stream->outbound_mixer);
 	if (stream->recorder_file) ms_free(stream->recorder_file);
+	if (stream->rtp_io_session) rtp_session_destroy(stream->rtp_io_session);
 
 	ms_free(stream);
 }
 
 static int dtmf_tab[16]={'0','1','2','3','4','5','6','7','8','9','*','#','A','B','C','D'};
 
-static void on_dtmf_received(RtpSession *s, unsigned long dtmf, void * user_data)
+static void on_dtmf_received(RtpSession *s, uint32_t dtmf, void *user_data)
 {
 	AudioStream *stream=(AudioStream*)user_data;
 	if (dtmf>15){
@@ -107,7 +112,7 @@ static void on_dtmf_received(RtpSession *s, unsigned long dtmf, void * user_data
  * more easily within the MSTicker API.
  * return TRUE if the decoder was changed, FALSE otherwise.
  */
-static bool_t audio_stream_payload_type_changed(RtpSession *session, unsigned long data) {
+static bool_t audio_stream_payload_type_changed(RtpSession *session, void *data) {
 	AudioStream *stream = (AudioStream *)data;
 	RtpProfile *prof = rtp_session_get_profile(session);
 	int payload = rtp_session_get_recv_payload_type(stream->ms.sessions.rtp_session);
@@ -125,16 +130,16 @@ static bool_t audio_stream_payload_type_changed(RtpSession *session, unsigned lo
 			ms_message("Ignore payload type change to CN");
 			return FALSE;
 		}
-		
+
 		if (stream->ms.current_pt && strcasecmp(pt->mime_type, stream->ms.current_pt->mime_type)==0 && pt->clock_rate==stream->ms.current_pt->clock_rate){
 			ms_message("Ignoring payload type number change because it points to the same payload type as the current one");
 			return FALSE;
 		}
-		
+
 		dec = ms_filter_create_decoder(pt->mime_type);
 		if (dec != NULL) {
 			MSFilter *nextFilter = stream->ms.decoder->outputs[0]->next.filter;
-			
+
 			ms_message("Replacing decoder on the fly");
 			ms_filter_unlink(stream->ms.rtprecv, 0, stream->ms.decoder, 0);
 			ms_filter_unlink(stream->ms.decoder, 0, nextFilter, 0);
@@ -143,7 +148,7 @@ static bool_t audio_stream_payload_type_changed(RtpSession *session, unsigned lo
 			stream->ms.decoder = dec;
 			configure_decoder(stream, pt, stream->sample_rate, stream->nchannels);
 			if (stream->write_resampler){
-				audio_stream_configure_resampler(stream->write_resampler,stream->ms.decoder,stream->soundwrite,pt);
+				audio_stream_configure_resampler(stream, stream->write_resampler,stream->ms.decoder,stream->soundwrite);
 			}
 			ms_filter_link(stream->ms.rtprecv, 0, stream->ms.decoder, 0);
 			ms_filter_link(stream->ms.decoder, 0, nextFilter, 0);
@@ -162,7 +167,7 @@ static bool_t audio_stream_payload_type_changed(RtpSession *session, unsigned lo
 /*
  * note: since not all filters implement MS_FILTER_GET_SAMPLE_RATE and MS_FILTER_GET_NCHANNELS, the PayloadType passed here is used to guess this information.
  */
-static void audio_stream_configure_resampler(MSFilter *resampler,MSFilter *from, MSFilter *to, PayloadType *pt) {
+static void audio_stream_configure_resampler(AudioStream *st, MSFilter *resampler,MSFilter *from, MSFilter *to) {
 	int from_rate=0, to_rate=0;
 	int from_channels = 0, to_channels = 0;
 	ms_filter_call_method(from,MS_FILTER_GET_SAMPLE_RATE,&from_rate);
@@ -170,20 +175,20 @@ static void audio_stream_configure_resampler(MSFilter *resampler,MSFilter *from,
 	ms_filter_call_method(from, MS_FILTER_GET_NCHANNELS, &from_channels);
 	ms_filter_call_method(to, MS_FILTER_GET_NCHANNELS, &to_channels);
 	if (from_channels == 0) {
-		from_channels = pt->channels;
+		from_channels = st->nchannels;
 		ms_error("Filter %s does not implement the MS_FILTER_GET_NCHANNELS method", from->desc->name);
 	}
 	if (to_channels == 0) {
-		to_channels = pt->channels;
+		to_channels = st->nchannels;
 		ms_error("Filter %s does not implement the MS_FILTER_GET_NCHANNELS method", to->desc->name);
 	}
 	if (from_rate == 0){
 		ms_error("Filter %s does not implement the MS_FILTER_GET_SAMPLE_RATE method", from->desc->name);
-		from_rate=pt->clock_rate;
+		from_rate = st->sample_rate;
 	}
 	if (to_rate == 0){
 		ms_error("Filter %s does not implement the MS_FILTER_GET_SAMPLE_RATE method", to->desc->name);
-		to_rate=pt->clock_rate;
+		to_rate = st->sample_rate;
 	}
 	ms_filter_call_method(resampler,MS_FILTER_SET_SAMPLE_RATE,&from_rate);
 	ms_filter_call_method(resampler,MS_FILTER_SET_OUTPUT_SAMPLE_RATE,&to_rate);
@@ -314,7 +319,7 @@ static void setup_local_player(AudioStream *stream, int samplerate, int channels
 	ms_filter_add_notify_callback(stream->local_player,player_callback,stream,TRUE);
 }
 
-static OrtpRtcpXrPlcStatus audio_stream_get_rtcp_xr_plc_status(unsigned long userdata) {
+static OrtpRtcpXrPlcStatus audio_stream_get_rtcp_xr_plc_status(void *userdata) {
 	AudioStream *stream = (AudioStream *)userdata;
 	if ((stream->features & AUDIO_STREAM_FEATURE_PLC) != 0) {
 		int decoder_have_plc = 0;
@@ -330,7 +335,7 @@ static OrtpRtcpXrPlcStatus audio_stream_get_rtcp_xr_plc_status(unsigned long use
 	return OrtpRtcpXrNoPlc;
 }
 
-static int audio_stream_get_rtcp_xr_signal_level(unsigned long userdata) {
+static int audio_stream_get_rtcp_xr_signal_level(void *userdata) {
 	AudioStream *stream = (AudioStream *)userdata;
 	if ((stream->features & AUDIO_STREAM_FEATURE_VOL_RCV) != 0) {
 		float volume = 0.f;
@@ -341,7 +346,7 @@ static int audio_stream_get_rtcp_xr_signal_level(unsigned long userdata) {
 	return ORTP_RTCP_XR_UNAVAILABLE_PARAMETER;
 }
 
-static int audio_stream_get_rtcp_xr_noise_level(unsigned long userdata) {
+static int audio_stream_get_rtcp_xr_noise_level(void *userdata) {
 	AudioStream *stream = (AudioStream *)userdata;
 	if ((stream->features & AUDIO_STREAM_FEATURE_VOL_RCV) != 0) {
 		float volume = 0.f;
@@ -352,19 +357,19 @@ static int audio_stream_get_rtcp_xr_noise_level(unsigned long userdata) {
 	return ORTP_RTCP_XR_UNAVAILABLE_PARAMETER;
 }
 
-static float audio_stream_get_rtcp_xr_average_quality_rating(unsigned long userdata) {
+static float audio_stream_get_rtcp_xr_average_quality_rating(void *userdata) {
 	AudioStream *stream = (AudioStream *)userdata;
 	return audio_stream_get_average_quality_rating(stream);
 }
 
-static float audio_stream_get_rtcp_xr_average_lq_quality_rating(unsigned long userdata) {
+static float audio_stream_get_rtcp_xr_average_lq_quality_rating(void *userdata) {
 	AudioStream *stream = (AudioStream *)userdata;
 	return audio_stream_get_average_lq_quality_rating(stream);
 }
 
 static bool_t ci_ends_with(const char *filename, const char*suffix){
-	int filename_len=strlen(filename);
-	int suffix_len=strlen(suffix);
+	size_t filename_len=strlen(filename);
+	size_t suffix_len=strlen(suffix);
 	if (filename_len<suffix_len) return FALSE;
 	return strcasecmp(filename+filename_len-suffix_len,suffix)==0;
 }
@@ -429,9 +434,9 @@ static void configure_av_player(AudioStream *stream, const MSFmtDescriptor *audi
 	struct _AVPlayer *player=&stream->av_player;
 	int stream_rate=0;
 	int stream_channels=0;
-	
+
 	ms_message("AudioStream [%p] Configure av_player, audiofmt=%s videofmt=%s",stream,ms_fmt_descriptor_to_string(audiofmt),ms_fmt_descriptor_to_string(videofmt));
-	
+
 	if (audiofmt){
 		if (player->decoder){
 			if (audiofmt->nchannels>0){
@@ -506,7 +511,7 @@ static int open_av_player(AudioStream *stream, const char *filename){
 		int channels=1;
 		ms_filter_call_method(player->player,MS_FILTER_GET_SAMPLE_RATE,&sr);
 		ms_filter_call_method(player->player,MS_FILTER_GET_NCHANNELS,&channels);
-		fmt1.fmt=ms_factory_get_audio_format(ms_factory_get_fallback(),"pcm", sr, channels, NULL);
+		fmt1.fmt=ms_factory_get_audio_format(stream->ms.factory, "pcm", sr, channels, NULL);
 		audiofmt=&fmt1;
 	}else{
 		if (fmt1.fmt && fmt1.fmt->type==MSAudio) {
@@ -586,7 +591,7 @@ static void setup_av_recorder(AudioStream *stream, int sample_rate, int nchannel
 			ms_filter_call_method(stream->av_recorder.resampler,MS_FILTER_SET_OUTPUT_SAMPLE_RATE,&g711_rate);
 			ms_filter_call_method(stream->av_recorder.resampler,MS_FILTER_SET_NCHANNELS,&nchannels);
 			ms_filter_call_method(stream->av_recorder.resampler,MS_FILTER_SET_OUTPUT_NCHANNELS,&g711_nchannels);
-			pinfmt.fmt=ms_factory_get_audio_format(ms_factory_get_fallback(),"pcmu",g711_rate,g711_nchannels,NULL);
+			pinfmt.fmt=ms_factory_get_audio_format(stream->ms.factory, "pcmu",g711_rate,g711_nchannels,NULL);
 
 		}else{
 			int got_sr=0;
@@ -597,7 +602,7 @@ static void setup_av_recorder(AudioStream *stream, int sample_rate, int nchannel
 			ms_filter_call_method(stream->av_recorder.resampler,MS_FILTER_SET_OUTPUT_SAMPLE_RATE,&got_sr);
 			ms_filter_call_method(stream->av_recorder.resampler,MS_FILTER_SET_NCHANNELS,&nchannels);
 			ms_filter_call_method(stream->av_recorder.resampler,MS_FILTER_SET_OUTPUT_NCHANNELS,&nchannels);
-			pinfmt.fmt=ms_factory_get_audio_format(ms_factory_get_fallback(),"opus",48000,nchannels,NULL);
+			pinfmt.fmt=ms_factory_get_audio_format(stream->ms.factory,"opus",48000,nchannels,NULL);
 		}
 		pinfmt.pin=1;
 		ms_message("Configuring av recorder with audio format %s",ms_fmt_descriptor_to_string(pinfmt.fmt));
@@ -685,12 +690,16 @@ static void setup_generic_confort_noise(AudioStream *stream){
 	RtpProfile *prof=rtp_session_get_profile(stream->ms.sessions.rtp_session);
 	PayloadType *pt=rtp_profile_get_payload(prof, rtp_session_get_send_payload_type(stream->ms.sessions.rtp_session));
 	PayloadType *cn=rtp_profile_find_payload(prof, "CN", 8000, 1);
-	
+
 	if (cn && pt && pt->channels==1 && pt->clock_rate==8000){
 		/* RFC3389 CN can be used*/
 		stream->vaddtx=ms_filter_new(MS_VAD_DTX_ID);
-		ms_filter_add_notify_callback(stream->vaddtx, on_silence_detected, stream, TRUE);
-		ms_filter_add_notify_callback(stream->ms.rtprecv, on_cn_received, stream, TRUE);
+		if (stream->vaddtx) {
+			ms_filter_add_notify_callback(stream->vaddtx, on_silence_detected, stream, TRUE);
+			ms_filter_add_notify_callback(stream->ms.rtprecv, on_cn_received, stream, TRUE);
+		} else {
+			ms_warning("Cannot instantiate vaddtx filter!");
+		}
 	}
 }
 
@@ -707,16 +716,38 @@ static void configure_decoder(AudioStream *stream, PayloadType *pt, int sample_r
 	}
 }
 
-int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char *rem_rtp_ip,int rem_rtp_port,
-	const char *rem_rtcp_ip, int rem_rtcp_port, int payload,int jitt_comp, const char *infile, const char *outfile,
-	MSSndCard *playcard, MSSndCard *captcard, bool_t use_ec){
+static int get_usable_telephone_event(RtpProfile *profile, int clock_rate){
+	int i;
+	int fallback_pt=-1;
+	for(i=0; i<128; ++i){
+		PayloadType *pt = profile->payload[i];
+		if (pt && strcasecmp(pt->mime_type, "telephone-event")==0 && (pt->flags & PAYLOAD_TYPE_FLAG_CAN_SEND)){
+			if (pt->clock_rate == clock_rate)
+				return i;
+			if (pt->clock_rate == 8000)
+				fallback_pt = i;
+		}
+	}
+	/*
+	 * the fallback payload type is used if the remote equipment doesn't conform to RFC4733 2.1,
+	 * that requires to use a clock rate which is the same as the audio codec.
+	 */
+	if (fallback_pt !=-1) ms_warning("The remote equipment doesn't conform to RFC4733 2.1 - it wants to use telephone-event/8000 despite the clock rate of the audio codec is %i", clock_rate);
+	return fallback_pt;
+}
+
+int audio_stream_start_from_io(AudioStream *stream, RtpProfile *profile, const char *rem_rtp_ip, int rem_rtp_port,
+	const char *rem_rtcp_ip, int rem_rtcp_port, int payload, const MSMediaStreamIO *io) {
 	RtpSession *rtps=stream->ms.sessions.rtp_session;
-	PayloadType *pt,*tel_ev;
-	int tmp;
+	PayloadType *pt;
+	int tmp, tev_pt;
 	MSConnectionHelper h;
 	int sample_rate;
 	int nchannels;
+	int err1,err2;
 	bool_t has_builtin_ec=FALSE;
+	
+	if (!ms_media_stream_io_is_consistent(io)) return -1;
 
 	rtp_session_set_profile(rtps,profile);
 	if (rem_rtp_port>0) rtp_session_set_remote_addr_full(rtps,rem_rtp_ip,rem_rtp_port,rem_rtcp_ip,rem_rtcp_port);
@@ -726,7 +757,6 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 		rtp_session_enable_rtcp(rtps, FALSE);
 	}
 	rtp_session_set_payload_type(rtps,payload);
-	rtp_session_set_jitter_compensation(rtps,jitt_comp);
 
 	if (rem_rtp_port>0)
 		ms_filter_call_method(stream->ms.rtpsend,MS_RTP_SEND_SET_SESSION,rtps);
@@ -734,31 +764,45 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	ms_filter_call_method(stream->ms.rtprecv,MS_RTP_RECV_SET_SESSION,rtps);
 	stream->ms.sessions.rtp_session=rtps;
 
-	if((stream->features & AUDIO_STREAM_FEATURE_DTMF) != 0)
+	if((stream->features & AUDIO_STREAM_FEATURE_DTMF_ECHO) != 0)
 		stream->dtmfgen=ms_filter_new(MS_DTMF_GEN_ID);
 	else
 		stream->dtmfgen=NULL;
 	rtp_session_signal_connect(rtps,"telephone-event",(RtpCallback)on_dtmf_received,stream);
 	rtp_session_signal_connect(rtps,"payload_type_changed",(RtpCallback)audio_stream_payload_type_changed,stream);
-	
+
 	if (stream->ms.state==MSStreamPreparing){
 		/*we were using the dummy preload graph, destroy it but keep sound filters unless no soundcard is given*/
-		_audio_stream_unprepare_sound(stream,captcard!=NULL);
+		_audio_stream_unprepare_sound(stream, io->input.type == MSResourceSoundcard);
 	}
-	
+
 	/* creates the local part */
-	if (captcard!=NULL){
+	if (io->input.type == MSResourceSoundcard){
 		if (stream->soundread==NULL)
-			stream->soundread=ms_snd_card_create_reader(captcard);
-		has_builtin_ec=!!(ms_snd_card_get_capabilities(captcard) & MS_SND_CARD_CAP_BUILTIN_ECHO_CANCELLER);
-	}else {
+			stream->soundread = ms_snd_card_create_reader(io->input.soundcard);
+		has_builtin_ec=!!(ms_snd_card_get_capabilities(io->input.soundcard) & MS_SND_CARD_CAP_BUILTIN_ECHO_CANCELLER);
+	} else if (io->input.type == MSResourceRtp) {
+		stream->rtp_io_session = io->input.session;
+		pt = rtp_profile_get_payload(rtp_session_get_profile(stream->rtp_io_session),
+			rtp_session_get_recv_payload_type(stream->rtp_io_session));
+		stream->soundread = ms_filter_new(MS_RTP_RECV_ID);
+		ms_filter_call_method(stream->soundread, MS_RTP_RECV_SET_SESSION, stream->rtp_io_session);
+		stream->read_decoder = ms_filter_create_decoder(pt->mime_type);
+	} else {
 		stream->soundread=ms_filter_new(MS_FILE_PLAYER_ID);
 		stream->read_resampler=ms_filter_new(MS_RESAMPLE_ID);
 	}
-	if (playcard!=NULL) {
+	if (io->output.type == MSResourceSoundcard) {
 		if (stream->soundwrite==NULL)
-			stream->soundwrite=ms_snd_card_create_writer(playcard);
-	}else {
+			stream->soundwrite=ms_snd_card_create_writer(io->output.soundcard);
+	} else if (io->output.type == MSResourceRtp) {
+		stream->rtp_io_session = io->output.session;
+		pt = rtp_profile_get_payload(rtp_session_get_profile(stream->rtp_io_session),
+			rtp_session_get_send_payload_type(stream->rtp_io_session));
+		stream->soundwrite = ms_filter_new(MS_RTP_SEND_ID);
+		ms_filter_call_method(stream->soundwrite, MS_RTP_SEND_SET_SESSION, stream->rtp_io_session);
+		stream->write_encoder = ms_filter_create_encoder(pt->mime_type);
+	} else {
 		stream->soundwrite=ms_filter_new(MS_FILE_REC_ID);
 	}
 
@@ -770,9 +814,9 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	}
 	nchannels=pt->channels;
 	stream->ms.current_pt=pt;
-	tel_ev=rtp_profile_get_payload_from_mime (profile,"telephone-event");
+	tev_pt = get_usable_telephone_event(profile, pt->clock_rate);
 
-	if ((stream->features & AUDIO_STREAM_FEATURE_DTMF_ECHO) != 0 && (tel_ev==NULL || ( (tel_ev->flags & PAYLOAD_TYPE_FLAG_CAN_RECV) && !(tel_ev->flags & PAYLOAD_TYPE_FLAG_CAN_SEND)))
+	if ((stream->features & AUDIO_STREAM_FEATURE_DTMF) != 0 && (tev_pt == -1)
 		&& ( strcasecmp(pt->mime_type,"pcmu")==0 || strcasecmp(pt->mime_type,"pcma")==0)){
 		/*if no telephone-event payload is usable and pcma or pcmu is used, we will generate
 		  inband dtmf*/
@@ -780,6 +824,8 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	} else {
 		stream->dtmfgen_rtp=NULL;
 	}
+	if (tev_pt != -1)
+		rtp_session_set_send_telephone_event_payload_type(rtps, tev_pt);
 
 	if (ms_filter_call_method(stream->ms.rtpsend,MS_FILTER_GET_SAMPLE_RATE,&sample_rate)!=0){
 		ms_error("Sample rate is unknown for RTP side !");
@@ -793,7 +839,7 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	/* be able to use the echo canceller wich may be limited (webrtc aecm max frequency is 16000 Hz) */
 	// First check if we need to use the echo canceller
 	// Overide feature if not requested or done at sound card level
-	if ( ((stream->features & AUDIO_STREAM_FEATURE_EC) && !use_ec) || has_builtin_ec )
+	if ( ((stream->features & AUDIO_STREAM_FEATURE_EC) && !stream->use_ec) || has_builtin_ec )
 		stream->features &=~AUDIO_STREAM_FEATURE_EC;
 
 	/*configure the echo canceller if required */
@@ -804,7 +850,7 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 
 	if ((stream->ms.encoder==NULL) || (stream->ms.decoder==NULL)){
 		/* big problem: we have not a registered codec for this payload...*/
-		ms_error("audio_stream_start_full: No decoder or encoder available for payload %s.",pt->mime_type);
+		ms_error("audio_stream_start_from_io: No decoder or encoder available for payload %s.",pt->mime_type);
 		return -1;
 	}
 
@@ -823,12 +869,20 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 				stream->ec = NULL;
 			}
 		}
+		/*in any case, our software AEC filters do not support stereo operation, so force channels to be 1*/
+		nchannels=1;
+	}
+	/*hack for opus, that claims stereo all the time, but we can't support stereo yet*/
+	if (strcasecmp(pt->mime_type,"opus")==0){
+		if ( (stream->features & (~AUDIO_STREAM_FEATURE_PLC) ) != 0){
+			/*all features except PLC prevent from activating the stereo*/
+			nchannels=1;
+		}else{
+			ms_message("Full stereo enabled in this audiostream.");
+		}
 	}
 	stream->sample_rate=sample_rate;
 	stream->nchannels=nchannels;
-	/*hack for opus, that claims stereo all the time, but we can't support stereo yet*/
-	if (strcasecmp(pt->mime_type,"opus")==0)
-		nchannels=1;
 
 	if ((stream->features & AUDIO_STREAM_FEATURE_VOL_SND) != 0)
 		stream->volsend=ms_filter_new(MS_VOLUME_ID);
@@ -838,14 +892,15 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 		stream->volrecv=ms_filter_new(MS_VOLUME_ID);
 	else
 		stream->volrecv=NULL;
+	
 	audio_stream_enable_echo_limiter(stream,stream->el_type);
 	audio_stream_enable_noise_gate(stream,stream->use_ng);
 
-	if (ms_filter_implements_interface(stream->soundread,MSFilterPlayerInterface) && infile){
-		audio_stream_play(stream,infile);
+	if (ms_filter_implements_interface(stream->soundread,MSFilterPlayerInterface) && io->input.file){
+		audio_stream_play(stream,io->input.file);
 	}
-	if (ms_filter_implements_interface(stream->soundwrite,MSFilterRecorderInterface) && outfile){
-		audio_stream_record(stream,outfile);
+	if (ms_filter_implements_interface(stream->soundwrite,MSFilterRecorderInterface) && io->output.file){
+		audio_stream_record(stream,io->output.file);
 	}
 
 	if (stream->use_agc){
@@ -863,22 +918,26 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 		ms_filter_call_method(stream->dtmfgen_rtp,MS_FILTER_SET_SAMPLE_RATE,&sample_rate);
 		ms_filter_call_method(stream->dtmfgen_rtp,MS_FILTER_SET_NCHANNELS,&nchannels);
 	}
+	
+	/*don't put these two statements in a single if, because the second one will not be executed if the first one evaluates as true*/
+	err1 = ms_filter_call_method(stream->soundread, MS_FILTER_SET_SAMPLE_RATE, &sample_rate);
+	err2 = ms_filter_call_method(stream->soundread, MS_FILTER_SET_NCHANNELS, &nchannels);
 	/* give the sound filters some properties */
-	if (ms_filter_call_method(stream->soundread,MS_FILTER_SET_SAMPLE_RATE,&sample_rate) != 0) {
+	if (err1 != 0 || err2 != 0){
 		/* need to add resampler*/
-		if (stream->read_resampler == NULL) stream->read_resampler=ms_filter_new(MS_RESAMPLE_ID);
+		if (stream->read_resampler == NULL) stream->read_resampler = ms_filter_new(MS_RESAMPLE_ID);
 	}
-	ms_filter_call_method(stream->soundread,MS_FILTER_SET_NCHANNELS,&nchannels);
 
-	if (ms_filter_call_method(stream->soundwrite,MS_FILTER_SET_SAMPLE_RATE,&sample_rate) != 0) {
+	err1 = ms_filter_call_method(stream->soundwrite, MS_FILTER_SET_SAMPLE_RATE, &sample_rate);
+	err2 = ms_filter_call_method(stream->soundwrite, MS_FILTER_SET_NCHANNELS, &nchannels);
+	if (err1 !=0 || err2 != 0){
 		/* need to add resampler*/
-		if (stream->write_resampler == NULL) stream->write_resampler=ms_filter_new(MS_RESAMPLE_ID);
+		if (stream->write_resampler == NULL) stream->write_resampler = ms_filter_new(MS_RESAMPLE_ID);
 	}
-	ms_filter_call_method(stream->soundwrite,MS_FILTER_SET_NCHANNELS,&nchannels);
 
 	if (stream->ec){
 		if (!stream->is_ec_delay_set) {
-			int delay_ms=ms_snd_card_get_minimal_latency(captcard);
+			int delay_ms=ms_snd_card_get_minimal_latency(io->input.soundcard);
 			ms_message("Setting echo canceller delay with value provided by soundcard: %i ms",delay_ms);
 			ms_filter_call_method(stream->ec,MS_ECHO_CANCELLER_SET_DELAY,&delay_ms);
 		} else {
@@ -904,7 +963,7 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 		ms_filter_call_method(stream->ms.encoder,MS_FILTER_SET_BITRATE,&stream->ms.target_bitrate);
 	}
 	rtp_session_set_target_upload_bandwidth(rtps, stream->ms.target_bitrate);
-	ms_filter_call_method(stream->ms.encoder,MS_FILTER_SET_NCHANNELS,&nchannels);	
+	ms_filter_call_method(stream->ms.encoder,MS_FILTER_SET_NCHANNELS,&nchannels);
 	if (pt->send_fmtp!=NULL) {
 		char value[16]={0};
 		int ptime;
@@ -916,7 +975,7 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 		}
 		ms_filter_call_method(stream->ms.encoder,MS_FILTER_ADD_FMTP, (void*)pt->send_fmtp);
 	}
-	
+
 	configure_decoder(stream, pt, sample_rate, nchannels);
 
 	/*create the equalizer*/
@@ -929,7 +988,7 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 		}
 	}else
 		stream->equalizer=NULL;
-	
+
 #ifdef ANDROID
 	/*configure equalizer if needed*/
 	audio_stream_set_mic_gain_db(stream, 0);
@@ -955,11 +1014,11 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 #endif
 
 	/*configure resamplers if needed*/
-	if (stream->read_resampler){
-		audio_stream_configure_resampler(stream->read_resampler, stream->soundread, stream->ms.encoder, pt);
+	if (stream->read_resampler) {
+		audio_stream_configure_resampler(stream, stream->read_resampler, stream->soundread, stream->ms.encoder);
 	}
-	if (stream->write_resampler){
-		audio_stream_configure_resampler(stream->write_resampler, stream->ms.decoder, stream->soundwrite, pt);
+	if (stream->write_resampler) {
+		audio_stream_configure_resampler(stream, stream->write_resampler, stream->ms.decoder, stream->soundwrite);
 	}
 
 	if (stream->ms.rc_enable){
@@ -1015,6 +1074,8 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	/*sending graph*/
 	ms_connection_helper_start(&h);
 	ms_connection_helper_link(&h,stream->soundread,-1,0);
+	if (stream->read_decoder)
+		ms_connection_helper_link(&h, stream->read_decoder, 0, 0);
 	if (stream->read_resampler)
 		ms_connection_helper_link(&h,stream->read_resampler,0,0);
 	if( stream->equalizer && stream->eq_loc == MSEqualizerMic )
@@ -1054,6 +1115,8 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 		ms_connection_helper_link(&h,stream->ec,0,0);
 	if (stream->write_resampler)
 		ms_connection_helper_link(&h,stream->write_resampler,0,0);
+	if (stream->write_encoder)
+		ms_connection_helper_link(&h, stream->write_encoder, 0, 0);
 	ms_connection_helper_link(&h,stream->soundwrite,0,-1);
 
 	/*call recording part, attached to both outgoing and incoming graphs*/
@@ -1075,9 +1138,40 @@ int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char
 	stream->ms.is_beginning=TRUE;
 	stream->ms.state=MSStreamStarted;
 
+	if (stream->soundwrite) {
+		if (ms_filter_implements_interface(stream->soundwrite, MSFilterAudioPlaybackInterface)) {
+			ms_filter_call_method(stream->soundwrite, MS_AUDIO_PLAYBACK_SET_ROUTE, &stream->audio_route);
+		}
+	}
+	
 	return 0;
 }
 
+int audio_stream_start_full(AudioStream *stream, RtpProfile *profile, const char *rem_rtp_ip,int rem_rtp_port,
+	const char *rem_rtcp_ip, int rem_rtcp_port, int payload,int jitt_comp, const char *infile, const char *outfile,
+	MSSndCard *playcard, MSSndCard *captcard, bool_t use_ec){
+	MSMediaStreamIO io = MS_MEDIA_STREAM_IO_INITIALIZER;
+	
+	if (playcard){
+		io.output.type = MSResourceSoundcard;
+		io.output.soundcard = playcard;
+	}else{
+		io.output.type = MSResourceFile;
+		io.output.file = outfile;
+	}
+	if (captcard){
+		io.input.type = MSResourceSoundcard;
+		io.input.soundcard = captcard;
+	}else{
+		io.input.type = MSResourceFile;
+		io.input.file = infile;
+	}
+	if (jitt_comp != -1)
+		rtp_session_set_jitter_compensation(stream->ms.sessions.rtp_session, jitt_comp);
+	audio_stream_enable_echo_canceller(stream, use_ec);
+	return audio_stream_start_from_io(stream, profile, rem_rtp_ip, rem_rtp_port, rem_rtcp_ip, rem_rtcp_port, payload, &io);
+}
+
 int audio_stream_start_with_files(AudioStream *stream, RtpProfile *prof,const char *remip, int remport,
 	int rem_rtcp_port, int pt,int jitt_comp, const char *infile, const char * outfile)
 {
@@ -1127,7 +1221,7 @@ void audio_stream_play(AudioStream *st, const char *name){
 		if (name != NULL) {
 			ms_filter_call_method(st->soundread,MS_FILE_PLAYER_OPEN,(void*)name);
 			if (st->read_resampler){
-				audio_stream_configure_resampler(st->read_resampler,st->soundread,st->ms.encoder, st->ms.current_pt);
+				audio_stream_configure_resampler(st, st->read_resampler,st->soundread,st->ms.encoder);
 			}
 			ms_filter_call_method_noarg(st->soundread,MS_FILE_PLAYER_START);
 		}
@@ -1171,7 +1265,7 @@ int audio_stream_mixed_record_open(AudioStream *st, const char* filename){
 
 static MSFilter *get_recorder(AudioStream *stream){
 	const char *fname=stream->recorder_file;
-	int len=strlen(fname);
+	size_t len=strlen(fname);
 
 	if (strstr(fname,".mkv")==fname+len-4){
 		if (stream->av_recorder.recorder){
@@ -1241,15 +1335,16 @@ AudioStream *audio_stream_new_with_sessions(const MSMediaStreamSessions *session
 		audio_stream_get_rtcp_xr_noise_level,
 		audio_stream_get_rtcp_xr_average_quality_rating,
 		audio_stream_get_rtcp_xr_average_lq_quality_rating,
-		(unsigned long)stream
+		stream
 	};
 
+	stream->ms.type = MSAudio;
+	stream->ms.sessions = *sessions;
+	media_stream_init(&stream->ms, ms_factory_get_fallback());
+
 	ms_filter_enable_statistics(TRUE);
 	ms_filter_reset_statistics();
 
-
-	stream->ms.type = MSAudio;
-	stream->ms.sessions=*sessions;
 	if (sessions->zrtp_context != NULL) {
 		ms_zrtp_set_stream_sessions(sessions->zrtp_context, &(stream->ms.sessions));
 	}
@@ -1268,8 +1363,6 @@ AudioStream *audio_stream_new_with_sessions(const MSMediaStreamSessions *session
 	}else{
 		stream->ec=ms_filter_new(MS_SPEEX_EC_ID);
 	}
-	stream->ms.evq=ortp_ev_queue_new();
-	rtp_session_register_event_queue(stream->ms.sessions.rtp_session,stream->ms.evq);
 	stream->play_dtmfs=TRUE;
 	stream->use_gc=FALSE;
 	stream->use_agc=FALSE;
@@ -1288,7 +1381,7 @@ AudioStream *audio_stream_new(int loc_rtp_port, int loc_rtcp_port, bool_t ipv6){
 AudioStream *audio_stream_new2(const char* ip, int loc_rtp_port, int loc_rtcp_port) {
 	AudioStream *obj;
 	MSMediaStreamSessions sessions={0};
-	sessions.rtp_session=create_duplex_rtpsession(ip,loc_rtp_port,loc_rtcp_port);
+	sessions.rtp_session=ms_create_duplex_rtp_session(ip,loc_rtp_port,loc_rtcp_port);
 	obj=audio_stream_new_with_sessions(&sessions);
 	obj->ms.owns_sessions=TRUE;
 	return obj;
@@ -1307,6 +1400,10 @@ void audio_stream_set_relay_session_id(AudioStream *stream, const char *id){
 	ms_filter_call_method(stream->ms.rtpsend, MS_RTP_SEND_SET_RELAY_SESSION_ID,(void*)id);
 }
 
+void audio_stream_enable_echo_canceller(AudioStream *st, bool_t enabled){
+	st->use_ec = enabled;
+}
+
 void audio_stream_set_echo_canceller_params(AudioStream *stream, int tail_len_ms, int delay_ms, int framesize){
 	if (stream->ec){
 		if (tail_len_ms>0)
@@ -1349,19 +1446,7 @@ void audio_stream_enable_noise_gate(AudioStream *stream, bool_t val){
 }
 
 void audio_stream_set_mic_gain_db(AudioStream *stream, float gain_db) {
-	float gain = gain_db;
-#ifdef ANDROID
-	SoundDeviceDescription *device = sound_device_description_get();
-	if (device && device->hacks) {
-		gain += device->hacks->mic_gain;
-		ms_message("Applying %f db to mic gain based on parameter and audio hack value in device table", gain);
-	}
-#endif
-
-	if (stream->volsend){
-		ms_filter_call_method(stream->volsend, MS_VOLUME_SET_DB_GAIN, &gain);
-	} else ms_warning("Could not apply gain: gain control wasn't activated. "
-			"Use audio_stream_enable_gain_control() before starting the stream.");
+	audio_stream_set_rtp_output_gain_db(stream, gain_db);
 }
 
 void audio_stream_set_mic_gain(AudioStream *stream, float gain){
@@ -1371,6 +1456,58 @@ void audio_stream_set_mic_gain(AudioStream *stream, float gain){
 			"Use audio_stream_enable_gain_control() before starting the stream.");
 }
 
+void audio_stream_set_sound_card_input_gain(AudioStream *stream, float volume) {
+	if (stream->soundread) {
+		if(ms_filter_implements_interface(stream->soundread, MSFilterAudioCaptureInterface)) {
+			ms_filter_call_method(stream->soundread, MS_AUDIO_CAPTURE_SET_VOLUME_GAIN, &volume);
+		}
+	} else {
+		ms_error("Cannot set input volume: no input filter");
+	}
+}
+
+float audio_stream_get_sound_card_input_gain(const AudioStream *stream) {
+	float volume;
+	
+	if(stream->soundread == NULL) {
+		ms_error("Cannot get input volume: no input filter");
+		return -1.0f;
+	}
+	if(!ms_filter_implements_interface(stream->soundread, MSFilterAudioCaptureInterface)) {
+		return -1.0f;
+	}
+	if(ms_filter_call_method(stream->soundread, MS_AUDIO_CAPTURE_GET_VOLUME_GAIN, &volume) < 0) {
+		volume = -1.0f;
+	}
+	return volume;
+}
+
+void audio_stream_set_sound_card_output_gain(AudioStream *stream, float volume) {
+	if (stream->soundwrite) {
+		if(ms_filter_implements_interface(stream->soundwrite, MSFilterAudioPlaybackInterface)) {
+			ms_filter_call_method(stream->soundwrite, MS_AUDIO_PLAYBACK_SET_VOLUME_GAIN, &volume);
+		}
+	} else {
+		ms_error("Cannot set output volume: no output filter");
+	}
+}
+
+float audio_stream_get_sound_card_output_gain(const AudioStream *stream) {
+	float volume;
+	
+	if(stream->soundwrite == NULL) {
+		ms_error("Cannot get output volume: no output filter");
+		return -1.0f;
+	}
+	if(!ms_filter_implements_interface(stream->soundwrite, MSFilterAudioPlaybackInterface)) {
+		return -1.0f;
+	}
+	if(ms_filter_call_method(stream->soundwrite, MS_AUDIO_PLAYBACK_GET_VOLUME_GAIN, &volume) < 0) {
+		volume = -1.0f;
+	}
+	return volume;
+}
+
 void audio_stream_enable_equalizer(AudioStream *stream, bool_t enabled){
 	stream->eq_active=enabled;
 	if (stream->equalizer){
@@ -1382,9 +1519,9 @@ void audio_stream_enable_equalizer(AudioStream *stream, bool_t enabled){
 void audio_stream_equalizer_set_gain(AudioStream *stream, int frequency, float gain, int freq_width){
 	if (stream->equalizer){
 		MSEqualizerGain d;
-		d.frequency=frequency;
+		d.frequency=(float)frequency;
 		d.gain=gain;
-		d.width=freq_width;
+		d.width=(float)freq_width;
 		ms_filter_call_method(stream->equalizer,MS_EQUALIZER_SET_GAIN,&d);
 	}
 }
@@ -1400,6 +1537,8 @@ static void dismantle_local_player(AudioStream *stream){
 }
 
 void audio_stream_stop(AudioStream * stream){
+	MSEventQueue *evq;
+
 	if (stream->ms.sessions.ticker){
 		MSConnectionHelper h;
 
@@ -1420,6 +1559,8 @@ void audio_stream_stop(AudioStream * stream){
 			/*dismantle the outgoing graph*/
 			ms_connection_helper_start(&h);
 			ms_connection_helper_unlink(&h,stream->soundread,-1,0);
+			if (stream->read_decoder != NULL)
+				ms_connection_helper_unlink(&h, stream->read_decoder, 0, 0);
 			if (stream->read_resampler!=NULL)
 				ms_connection_helper_unlink(&h,stream->read_resampler,0,0);
 			if( stream->equalizer && stream->eq_loc == MSEqualizerMic)
@@ -1459,6 +1600,8 @@ void audio_stream_stop(AudioStream * stream){
 				ms_connection_helper_unlink(&h,stream->ec,0,0);
 			if (stream->write_resampler!=NULL)
 				ms_connection_helper_unlink(&h,stream->write_resampler,0,0);
+			if (stream->write_encoder != NULL)
+				ms_connection_helper_unlink(&h, stream->write_encoder, 0, 0);
 			ms_connection_helper_unlink(&h,stream->soundwrite,0,-1);
 
 			/*dismantle the call recording */
@@ -1476,6 +1619,10 @@ void audio_stream_stop(AudioStream * stream){
 	rtp_session_set_rtcp_xr_media_callbacks(stream->ms.sessions.rtp_session, NULL);
 	rtp_session_signal_disconnect_by_callback(stream->ms.sessions.rtp_session,"telephone-event",(RtpCallback)on_dtmf_received);
 	rtp_session_signal_disconnect_by_callback(stream->ms.sessions.rtp_session,"payload_type_changed",(RtpCallback)audio_stream_payload_type_changed);
+	/*before destroying the filters, pump the event queue so that pending events have a chance to reach their listeners.
+	 * When the filter are destroyed, all their pending events in the event queue will be cancelled*/
+	evq = ms_factory_get_event_queue(stream->ms.factory);
+	if (evq) ms_event_queue_pump(evq);
 	audio_stream_free(stream);
 	ms_filter_log_statistics();
 }
@@ -1489,6 +1636,22 @@ int audio_stream_send_dtmf(AudioStream *stream, char dtmf)
 	return 0;
 }
 
+static void audio_stream_set_rtp_output_gain_db(AudioStream *stream, float gain_db) {
+	float gain = gain_db;
+#ifdef ANDROID
+	SoundDeviceDescription *device = sound_device_description_get();
+	if (device && device->hacks) {
+		gain += device->hacks->mic_gain;
+		ms_message("Applying %f db to mic gain based on parameter and audio hack value in device table", gain);
+	}
+#endif
+
+	if (stream->volsend){
+		ms_filter_call_method(stream->volsend, MS_VOLUME_SET_DB_GAIN, &gain);
+	} else ms_warning("Could not apply gain: gain control wasn't activated. "
+			"Use audio_stream_enable_gain_control() before starting the stream.");
+}
+
 void audio_stream_mute_rtp(AudioStream *stream, bool_t val)
 {
 	if (stream->ms.rtpsend){
@@ -1526,13 +1689,6 @@ bool_t audio_stream_zrtp_enabled(const AudioStream *stream) {
 	return stream->ms.sessions.zrtp_context!=NULL;
 }
 
-void audio_stream_enable_dtls(AudioStream *stream, MSDtlsSrtpParams *params){
-	if (stream->ms.sessions.dtls_context==NULL) {
-		ms_message("Start DTLS audio stream context in stream sessions [%p]", &(stream->ms.sessions));
-		stream->ms.sessions.dtls_context=ms_dtls_srtp_context_new(&(stream->ms.sessions), params);
-	}
-}
-
 static void configure_av_recorder(AudioStream *stream){
 	if (stream->av_recorder.video_input && stream->av_recorder.recorder){
 		MSPinFormat pinfmt={0};
@@ -1547,19 +1703,26 @@ static void configure_av_recorder(AudioStream *stream){
 
 void audio_stream_link_video(AudioStream *stream, VideoStream *video){
 	stream->videostream=video;
-	if (stream->av_recorder.video_input && video->itcsink){
+	if (stream->av_recorder.video_input && video->recorder_output){
 		ms_message("audio_stream_link_video() connecting itc filters");
-		ms_filter_call_method(video->itcsink,MS_ITC_SINK_CONNECT,stream->av_recorder.video_input);
+		ms_filter_call_method(video->recorder_output,MS_ITC_SINK_CONNECT,stream->av_recorder.video_input);
 		configure_av_recorder(stream);
 	}
 }
 
 void audio_stream_unlink_video(AudioStream *stream, VideoStream *video){
 	stream->videostream=NULL;
-	if (stream->av_recorder.video_input && video->itcsink){
-		ms_filter_call_method(video->itcsink,MS_ITC_SINK_CONNECT,NULL);
+	if (stream->av_recorder.video_input && video->recorder_output){
+		ms_filter_call_method(video->recorder_output,MS_ITC_SINK_CONNECT,NULL);
 	}
 }
 
-
+void audio_stream_set_audio_route(AudioStream *stream, MSAudioRoute route) {
+	stream->audio_route = route;
+	if (stream->soundwrite) {
+		if (ms_filter_implements_interface(stream->soundwrite, MSFilterAudioPlaybackInterface)) {
+			ms_filter_call_method(stream->soundwrite, MS_AUDIO_PLAYBACK_SET_ROUTE, &route);
+		}
+	}
+}
 
