@@ -26,10 +26,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2_tester.h"
 #include "mediastreamer2_tester_private.h"
 #include "qosanalyzer.h"
-
-#include <stdio.h>
-#include "CUnit/Basic.h"
-
+#include <math.h>
 
 static RtpProfile rtp_profile;
 
@@ -44,7 +41,7 @@ static RtpProfile rtp_profile;
 #define EDGE_BW 10
 #define THIRDGENERATION_BW 200
 
-static int tester_init(void) {
+static int tester_before_all(void) {
 	ms_init();
 	ms_filter_enable_statistics(TRUE);
 	ortp_init();
@@ -60,7 +57,7 @@ static int tester_init(void) {
 	return 0;
 }
 
-static int tester_cleanup(void) {
+static int tester_after_all(void) {
 	ortp_exit();
 	ms_exit();
 	rtp_profile_clear_all(&rtp_profile);
@@ -139,19 +136,20 @@ static void audio_manager_start(stream_manager_t * mgr
 
 	media_stream_set_target_network_bitrate(&mgr->audio_stream->ms,target_bitrate);
 
-	CU_ASSERT_EQUAL(audio_stream_start_full(mgr->audio_stream
-												, &rtp_profile
-												, "127.0.0.1"
-												, remote_port
-												, "127.0.0.1"
-												, remote_port+1
-												, payload_type
-												, 50
-												, player_file
-												, recorder_file
-												, NULL
-												, NULL
-												, 0),0);
+	BC_ASSERT_EQUAL(audio_stream_start_full(mgr->audio_stream
+											, &rtp_profile
+											, "127.0.0.1"
+											, remote_port
+											, "127.0.0.1"
+											, remote_port+1
+											, payload_type
+											, 50
+											, player_file
+											, recorder_file
+											, NULL
+											, NULL
+											, 0)
+					,0, int, "%d");
 }
 
 #if VIDEO_ENABLED
@@ -172,7 +170,7 @@ static void video_manager_start(	stream_manager_t * mgr
 												, payload_type
 												, 60
 												, cam);
-	CU_ASSERT_EQUAL(result,0);
+	BC_ASSERT_EQUAL(result,0, int, "%d");
 }
 #endif
 
@@ -202,8 +200,8 @@ void start_adaptive_stream(MSFormatType type, stream_manager_t ** pmarielle, str
 	stream_manager_t *marielle=*pmarielle=stream_manager_new(type);
 	stream_manager_t *margaux=*pmargaux=stream_manager_new(type);
 
-	char* file = ms_strdup_printf("%s/%s", bc_tester_read_dir_prefix, HELLO_16K_1S_FILE);
-	char* recorded_file = ms_strdup_printf("%s/%s", bc_tester_writable_dir_prefix, RECORDED_16K_1S_FILE);
+	char* file = bc_tester_res(HELLO_16K_1S_FILE);
+	char* recorded_file = bc_tester_file(RECORDED_16K_1S_FILE);
 
 	marielle->user_data = recorded_file;
 	params.enabled=TRUE;
@@ -221,7 +219,7 @@ void start_adaptive_stream(MSFormatType type, stream_manager_t ** pmarielle, str
 
 	/* Disable avpf. */
 	pt = rtp_profile_get_payload(&rtp_profile, VP8_PAYLOAD_TYPE);
-	CU_ASSERT_PTR_NOT_NULL_FATAL(pt);
+	BC_ASSERT_PTR_NOT_NULL_FATAL(pt);
 	payload_type_unset_flag(pt, PAYLOAD_TYPE_RTCP_FEEDBACK_ENABLED);
 
 
@@ -238,7 +236,7 @@ void start_adaptive_stream(MSFormatType type, stream_manager_t ** pmarielle, str
 #if VIDEO_ENABLED
 		marielle->video_stream->staticimage_webcam_fps_optimization = FALSE;
 		video_manager_start(marielle,payload,margaux->local_rtp,0,marielle_webcam);
-		video_stream_set_direction(margaux->video_stream,VideoStreamRecvOnly);
+		video_stream_set_direction(margaux->video_stream, MediaStreamRecvOnly);
 		video_manager_start(margaux,payload,marielle->local_rtp,0,NULL);
 #else
 		ms_fatal("Unsupported stream type [%s]",ms_format_type_to_string(marielle->type));
@@ -250,9 +248,8 @@ void start_adaptive_stream(MSFormatType type, stream_manager_t ** pmarielle, str
 						*pmarielle);
 	rtp_session_enable_network_simulation(margaux_ms->sessions.rtp_session,&params);
 
-	ms_free(recorded_file);
-	ms_free(file);
-
+	free(recorded_file);
+	free(file);
 }
 
 static void iterate_adaptive_stream(stream_manager_t * marielle, stream_manager_t * margaux,
@@ -268,17 +265,17 @@ static void iterate_adaptive_stream(stream_manager_t * marielle, stream_manager_
 		margaux_ms=&margaux->video_stream->ms;
 	}
 
-	while ((!current||*current<expected) && retry++ <timeout_ms/100) {
+	while ((!current||*current<expected) && retry++/5 <timeout_ms/100) {
 		media_stream_iterate(marielle_ms);
 		media_stream_iterate(margaux_ms);
 		// handle_queue_events(marielle);
-		if (retry%10==0) {
+		if (retry%50==0) {
 			 ms_message("stream [%p] bandwidth usage: [d=%.1f,u=%.1f] kbit/sec"	,
 				marielle_ms, media_stream_get_down_bw(marielle_ms)/1000, media_stream_get_up_bw(marielle_ms)/1000);
 			 ms_message("stream [%p] bandwidth usage: [d=%.1f,u=%.1f] kbit/sec"	,
 				margaux_ms, media_stream_get_down_bw(margaux_ms)/1000, media_stream_get_up_bw(margaux_ms)/1000);
 		 }
-		ms_usleep(100000);
+		ms_usleep(20000);
 	}
 }
 
@@ -310,8 +307,8 @@ static void event_queue_cb(MediaStream *ms, void *user_pointer) {
 
 					if (rb&&ortp_loss_rate_estimator_process_report_block(ctx->estimator,&ms->sessions.rtp_session->rtp,rb)){
 						float diff = fabs(ortp_loss_rate_estimator_get_value(ctx->estimator) - ctx->loss_rate);
-						CU_ASSERT_TRUE(diff >= 0);
-						CU_ASSERT_TRUE(diff <= 10);
+						BC_ASSERT_TRUE(diff >= 0);
+						BC_ASSERT_TRUE(diff <= 10);
 					}
 				} while (rtcp_next_packet(evd->packet));
 			}
@@ -331,11 +328,11 @@ static void packet_duplication() {
 	media_stream_enable_adaptive_bitrate_control(&marielle->audio_stream->ms,FALSE);
 	iterate_adaptive_stream(marielle, margaux, 10000, NULL, 0);
 	stats=rtp_session_get_stats(margaux->video_stream->ms.sessions.rtp_session);
-	CU_ASSERT_EQUAL(stats->packet_dup_recv, dup_ratio ? stats->packet_recv / (dup_ratio+1) : 0);
+	BC_ASSERT_EQUAL(stats->packet_dup_recv, dup_ratio ? stats->packet_recv / (dup_ratio+1) : 0, int, "%d");
 	/*in theory, cumulative loss should be the invert of duplicated count, but
 	since cumulative loss is computed only on received RTCP report and duplicated
 	count is updated on each RTP packet received, we cannot accurately compare these values*/
-	CU_ASSERT_TRUE(stats->cum_packet_loss <= -.5*stats->packet_dup_recv);
+	BC_ASSERT_TRUE(stats->cum_packet_loss <= -.5*stats->packet_dup_recv);
 	stop_adaptive_stream(marielle,margaux);
 
 	dup_ratio = 1;
@@ -343,8 +340,8 @@ static void packet_duplication() {
 	media_stream_enable_adaptive_bitrate_control(&marielle->audio_stream->ms,FALSE);
 	iterate_adaptive_stream(marielle, margaux, 10000, NULL, 0);
 	stats=rtp_session_get_stats(margaux->video_stream->ms.sessions.rtp_session);
-	CU_ASSERT_EQUAL(stats->packet_dup_recv, dup_ratio ? stats->packet_recv / (dup_ratio+1) : 0);
-	CU_ASSERT_TRUE(stats->cum_packet_loss <= -.5*stats->packet_dup_recv);
+	BC_ASSERT_EQUAL(stats->packet_dup_recv, dup_ratio ? stats->packet_recv / (dup_ratio+1) : 0, int, "%d");
+	BC_ASSERT_TRUE(stats->cum_packet_loss <= -.5*stats->packet_dup_recv);
 	stop_adaptive_stream(marielle,margaux);
 }
 
@@ -361,7 +358,7 @@ static void upload_bandwidth_computation() {
 			rtp_session_set_duplication_ratio(marielle->audio_stream->ms.sessions.rtp_session, i);
 			iterate_adaptive_stream(marielle, margaux, 5000, NULL, 0);
 			/*since PCMA uses 80kbit/s, upload bandwidth should just be 80+80*duplication_ratio kbit/s */
-			CU_ASSERT_TRUE(fabs(rtp_session_get_send_bandwidth(marielle->audio_stream->ms.sessions.rtp_session)/1000. - 80.*(i+1)) < 5.f);
+			BC_ASSERT_TRUE(fabs(rtp_session_get_send_bandwidth(marielle->audio_stream->ms.sessions.rtp_session)/1000. - 80.*(i+1)) < 5.f);
 		}
 		stop_adaptive_stream(marielle,margaux);
 	}
@@ -405,8 +402,8 @@ void upload_bitrate(const char* codec, int payload, int target_bw, int expect_bw
 		media_stream_enable_adaptive_bitrate_control(&marielle->audio_stream->ms,FALSE);
 		iterate_adaptive_stream(marielle, margaux, 15000, NULL, 0);
 		upload_bw=media_stream_get_up_bw(&marielle->audio_stream->ms) / 1000;
-		CU_ASSERT_TRUE(upload_bw >= expect_bw-2);
-		CU_ASSERT_TRUE(upload_bw <= expect_bw+2);
+		BC_ASSERT_TRUE(upload_bw >= expect_bw-2);
+		BC_ASSERT_TRUE(upload_bw <= expect_bw+2);
 		stop_adaptive_stream(marielle,margaux);
 	}
 }
@@ -451,10 +448,10 @@ void adaptive_video(int max_bw, int exp_min_bw, int exp_max_bw, int loss_rate, i
 		stream_manager_t * marielle, * margaux;
 		start_adaptive_stream(MSVideo, &marielle, &margaux, VP8_PAYLOAD_TYPE, 300*1000, max_bw*1000, loss_rate, 50,0);
 		iterate_adaptive_stream(marielle, margaux, 100000, &marielle->rtcp_count, 7);
-		CU_ASSERT_TRUE(marielle->adaptive_stats.loss_estim >= exp_min_loss);
-		CU_ASSERT_TRUE(marielle->adaptive_stats.loss_estim <= exp_max_loss);
-		CU_ASSERT_TRUE(marielle->adaptive_stats.congestion_bw_estim >= exp_min_bw);
-		CU_ASSERT_TRUE(marielle->adaptive_stats.congestion_bw_estim <= exp_max_bw);
+		BC_ASSERT_TRUE(marielle->adaptive_stats.loss_estim >= exp_min_loss);
+		BC_ASSERT_TRUE(marielle->adaptive_stats.loss_estim <= exp_max_loss);
+		BC_ASSERT_TRUE(marielle->adaptive_stats.congestion_bw_estim >= exp_min_bw);
+		BC_ASSERT_TRUE(marielle->adaptive_stats.congestion_bw_estim <= exp_max_bw);
 		stop_adaptive_stream(marielle,margaux);
 	}
 }
@@ -496,8 +493,10 @@ static test_t tests[] = {
 
 test_suite_t adaptive_test_suite = {
 	"AdaptiveAlgorithm",
-	tester_init,
-	tester_cleanup,
+	tester_before_all,
+	tester_after_all,
+	NULL,
+	NULL,
 	sizeof(tests) / sizeof(tests[0]),
 	tests
 };
